/******************************************************************************
 *
 *                Copyright (c) 2017-2019 by Löwenware Ltd
 *             Please, refer LICENSE file for legal information
 *
 ******************************************************************************/

/**
 * @file buffer.h
 * @author Ilja Kartašov <ik@lowenware.com>
 * @brief Buffer module header file
 *
 * @see https://lowenware.com/aisl/
 */

#ifndef AISL_BUFFER_H_D60EB5DF_B70B_4D8F_AA63_7FDB569D67E9
#define AISL_BUFFER_H_D60EB5DF_B70B_4D8F_AA63_7FDB569D67E9

#include <stdint.h>
#include <stdarg.h>


struct buffer
{
  char    * data;
  int32_t   size;
  int32_t   used;
};

typedef struct buffer * buffer_t;


int32_t
buffer_init( buffer_t buffer, int32_t size );


void
buffer_release( buffer_t buffer );


int32_t
buffer_insert(buffer_t buffer, int32_t offset, const char * data, int32_t size);


int32_t
buffer_append_printf( buffer_t buffer, const char * format, ... );


int32_t
buffer_append_vprintf( buffer_t buffer, const char * format, va_list args );


int32_t
buffer_append( buffer_t buffer, const char * data, int32_t size );


int32_t
buffer_shift( buffer_t buffer, int32_t size );



#endif /* !AISL_BUFFER_H */
