#ifndef AISL_CLIENT_H_164FE6B2_E5D4_4968_B50F_823E30E8F777
#define AISL_CLIENT_H_164FE6B2_E5D4_4968_B50F_823E30E8F777

#include <arpa/inet.h>

#include <aisl/client.h>
#include "buffer.h"

#define AISL_CLIENT(x) ((aisl_client_t) x)


struct aisl_client
{
  struct sockaddr_in   address;       /**< Client's address structure. */
  aisl_server_t        server;        /**< Server instance. */
  struct buffer        in;            /**< Client's input buffer. */
  struct buffer        out;           /**< Client's output buffer. */
  #ifndef AISL_WITHOUT_SSL
  SSL                * ssl;           /**< SSL pointer for HTTPS. */
  #endif
  time_t               timestamp;     /**< Last communication timestamp. */

  aisl_stream_t        stream;        /**< Pending client's stream. */
  int                  next_id;       /**< Stream id generator (even). */
  int                  flags;         /**< Client's flag bitmask. */
  int                  fd;            /**< Client's socket descriptor. */

  aisl_http_version_t  http_version;  /**< Client's http_version version. */
};


/**
 * @brief Constructor for #aisl_client_t instance.
 * @param server an #aisl_server_t instance pointer.
 * @param fd a client socket descriptor.
 * @param addr a pointer to client's address structure.
 * @param ssl_ctx a pointer to SSL context or NULL if encryption is disabled
 */
aisl_client_t
aisl_client_new( aisl_server_t        server,
                 int                  fd,
                 struct sockaddr_in * addr );


/**
 * @brief Destructor for #aisl_client_t instance.
 * @param client an #aisl_client_t instance pointer.
 */
void
aisl_client_free(aisl_client_t client);


/**
 * @brief Does all HTTP client routines.
 * Reads and parses requests, writes responses.
 * @param client an #aisl_client_t instance pointer.
 * @param timeout an allowed client silence time in seconds.
 * @return #aisl_status_t code.
 */
aisl_status_t
aisl_client_touch(aisl_client_t client, int32_t timeout);


/**
 * @Brief Checks if client is about to keep connection alive.
 * @param client an #aisl_client_t instance pointer.
 * @return true if keepalive mode is on, otherwise false.
 */
bool
aisl_client_get_keepalive(aisl_client_t client);


/**
 * @Brief Sets if connection with client must be kept alive.
 * @param client an #aisl_client_t instance pointer.
 * @param value a true to enable keepalive mode, false to disable.
 */
void
aisl_client_set_keepalive(aisl_client_t client, bool value);


/**
 * @brief Gets socket descriptor associated with #aisl_client_t instance.
 * @param client an #aisl_client_t instance pointer.
 * @return a client socket descriptor.
 */
int
aisl_client_get_socket(aisl_client_t client);


#endif /* !AISL_CLIENT_H */
